PROC GLOBAL

set explicit

numeric minlat maxlat minlong maxlong xlat xlong mindi cln dd err ss;

PROC GPS_FF
preproc
{ !!! UPDATE THESE VALUES FOR YOUR COUNTRY !!! }
minlat = -20;       { minimum latitude in the country - negative = South, positive = North }
maxlat = 50;        { maximum latitude in the country - negative = South, positive = North }
minlong = -20;      { minimum longitude in the country - negative = West, positive = East }
maxlong = 50;       { maximum longitude in the country - negative = West, positive = East }
mindi = 20050401;   { earliest interview date in yyyymmdd format } 

PROC GP1
{ check cluster is in the sample file }
recode GP1      => err;
       001-799  => 0;    { Give list of valid cluster numbers }
                => 1;
endrecode;
if err then
  display("Cluster number not valid");
  reenter;
endif;

PROC GP2
{ !!! ENTER CORRESPONDENCE BETWEEN CLUSTER AND AREA FOR YOUR COUNTRY !!! }
recode GP1  : GP2 => err;
    001-100 :  1  => 0;
    101-200 :  2  => 0;
    201-300 :  1  => 0;
    301-400 :  2  => 0;
    401-500 :  1  => 0;
    501-600 :  2  => 0;
    601-700 :  1  => 0;
    701-800 :  2  => 0;
            :     => 1;
endrecode;
if err then
  display("Cluster identification is incorrect");
  reenter
endif;

PROC GP3
{ !!! ENTER CORRESPONDENCE BETWEEN CLUSTER AND REGION FOR YOUR COUNTRY !!! }
recode GP1  : GP3 => err;
    001-100 :  1  => 0;
    101-200 :  1  => 0;
    201-300 :  2  => 0;
    301-400 :  2  => 0;
    401-500 :  3  => 0;
    501-600 :  3  => 0;
    601-700 :  4  => 0;
    701-800 :  4  => 0;
            :     => 1;
endrecode;
if err then
  display("Cluster identification is incorrect");
  reenter
endif;

PROC GP5Y
recode GP5Y  :     GP5M        : GP5D => err;
   2005-2006 : 1,3,5,7,8,10,12 : 1-31 => 0;
   2005-2006 : 4,6,9,11        : 1-30 => 0;
   2005-2006 : 2               : 1-28 => 0;
             :                 :      => 1;
endrecode;
if err then
  display("Date of interview impossible");
  reenter
endif;
dd = (GP5Y*100 + GP5M)*100 + GP5D;
ss = sysdate("yyyymmdd");
if dd >= ss or dd < mindi then
  display("Date of interview impossible");
  reenter
endif;

PROC GP7NS
if GP7NS <> "N" & GP7NS <> "S" then
  display("You must enter N for North or S for South.");
  reenter;
endif;

PROC GP7D
if GP7NS = "N" then
  xlat = GP7D;
elseif GP7NS = "S" then
  xlat = (0-GP7D);
endif;  
if not xlat in minlat:maxlat then
  if maxlat < 0 then
    display("The waypoint's latitude is outside the acceptable range (%dS:%dS)",(0-minlat),(0-maxlat));
  elseif minlat < 0 then
    display("The waypoint's latitude is outside the acceptable range (%dS:%dN)",(0-minlat),maxlat);
  else  
    display("The waypoint's latitude is outside the acceptable range (%dN:%dN)",minlat,maxlat);
  endif;
endif;

PROC GP8EW
if GP8EW <> "E" & GP8EW <> "W" then
  display("You must enter E for East or W for West.");
  reenter;
endif;

PROC GP8D
if GP8EW = "E" then
  xlong = GP8D;
elseif GP8EW = "W" then
  xlong = (0-GP8D);
endif;  
if not xlong in minlong:maxlong then
  if maxlong < 0 then
    display("The waypoint's longitude is outside the acceptable range (%dW:%dW)",(0-minlong),(0-maxlong));
  elseif minlong < 0 then
    display("The waypoint's longitude is outside the acceptable range (%dW:%dE)",(0-minlong),maxlong);
  else  
    display("The waypoint's longitude is outside the acceptable range (%dE:%dE)",minlong,maxlong);
  endif;
endif;

PROC DP6
if DP6 <> GP6 then
  display("Different waypoint names entered! Check entered values against the questionnaire");
  DP6 = "";
  reenter;
endif;

PROC DP7NS
if DP7NS <> GP7NS then
  display("Different latitudinal orientations entered! Check entered values against the questionnaire");
  DP7NS = "";
  reenter;
endif;

PROC DP7D
if DP7D <> GP7D then
  display("Different latitudinal decimal degrees entered! Check entered values against the questionnaire");
  DP7D = 0;
  reenter;
endif;

PROC DP8EW
if DP8EW <> GP8EW then
  display("Different longitudinal orientations entered! Check entered values against the questionnaire");
  DP8EW = "";
  reenter;
endif;

PROC DP8D
if DP8D <> GP8D then
  display("Different longitudinal decimal degrees entered! Check entered values against the questionnaire");
  DP8D = 0;
  reenter;
endif;

