PROC GLOBAL
set explicit;
{ Working variables -- check the lengths needed for all fields }
alpha(32) CSPro;			{ CSPro executables directory }
alpha(2)  cc;				{ country code }
alpha(26)  proj;				{ project directory name }
alpha(8)  app, dic;			{ application name and dictionary name }
alpha(8)  opid, nl;			{ operator ID, "newline" string }
alpha(30)  floppy, wrkdrv;               { floppy and workdrive letters, e.g. "A:", "C:" }
alpha(32) wrkprj, data, veri, xdir;     { directories }
alpha(8)  entry, dicts;                 { directory names for ENTRY, DICTS -- suggest using these names }
alpha(3)  xcluster;                     { cluster number -- make sure length is correct!!! }
alpha(2)  fcodes;                       { file codes - M = Main, V = Verification }
alpha(12) ocodes;                       { option codes AMT-RUV-NDQ }
alpha(2)  hcodes;                       { code for household data file name }

numeric always, choice, chkok, err;

file pfffile mdata vdata datacopy jstone dirname;

{ ---------------------------------------------------------------------------- }

function chkdir()
  setfile(dirname,xdir);
  if fileexist(xdir) <> 1 then
    errmsg("Directory %s does not exist",xdir);
    stop(-1);
  endif;
end;

function chkexist(addmode,mainver)
  chkok = 1;
  if mainver = 1 then {if main data}
    if !fileexist(mdata) & !addmode then {main data doesn't exist and in modify mode}
      errmsg("     %s   -   A",xcluster);
      chkok = 0;
    endif;
  else {if verification data}
    if !fileexist(vdata) & !addmode then {verification data doesn't exist and in modify mode}
      errmsg("Verification data file for cluster %s does not exist  - use Option %s",xcluster,"R");
      chkok = 0;
    endif;
    if chkok then {if verification data file exists on this machine...}
      if fileexist(mdata) then {...AND main data file exists on this machine}
        errmsg("Cluster %s can not be verified in this machine",xcluster);
        chkok = 0; {prevent data entry}
      endif;
    endif;
  endif;
  chkexist = chkok;
end;

function genpff(addmode,mainver)
  filecreate(pfffile);
  filewrite(pfffile, "[Run Information]");
  filewrite(pfffile, "Version=CSPro 2.6");
  filewrite(pfffile, "AppType=Entry");
  filewrite(pfffile, "");
  filewrite(pfffile, "[DataEntryInit]");
  filewrite(pfffile, concat("OperatorID=", strip(opid)));
  if addmode then
    filewrite(pfffile, "StartMode=ADD");
    filewrite(pfffile, "Lock=Verify");
    filewrite(pfffile, "Fullscreen=Yes");
  else
    { no StartMode for modify }
    filewrite(pfffile, "Lock=Add,Verify");
  endif;
  filewrite(pfffile, "NoFileOpen=Yes");
  filewrite(pfffile, "");
  filewrite(pfffile, "[Files]");
  filewrite(pfffile, concat("Application=", strip(wrkprj), "\", strip(entry), "\", strip(app), ".ent"));
  if mainver = 1 then
    filewrite(pfffile, concat("InputData=", strip(data), "\", hcodes, xcluster, fcodes[mainver:1], ".DAT"));
  else
    filewrite(pfffile, concat("InputData=", strip(veri), "\", hcodes, xcluster, fcodes[mainver:1], ".DAT"));
  endif;
  filewrite(pfffile, "");
  filewrite(pfffile, "[Parameters]");
  filewrite(pfffile, concat("Parameter=", xcluster));
  close(pfffile);
end;

function runpff();
  { Using single quotes instad of double quotes as double quotes are needed in the string generated }
  Execsystem(concat('"', strip(cspro), '\CSEntry.exe" ',
                    strip(wrkprj), "\", strip(entry), "\", strip(app), '.pff'),
             maximized,wait); 
end;

function csrun(addmode,mainver)
  if chkexist(addmode,mainver) then
    genpff(addmode,mainver);
    runpff();
  { backupf(mainver); -- not used for now }
  endif;
end;

function transfer(mainver)
  { !!! NETWORK !!! remove message if using a network}
  errmsg("***     ?",floppy);
  setfile(datacopy,concat(strip(floppy), "\",hcodes,xcluster, fcodes[mainver  :1],  ".dat"));
  if mainver = 1 then { if the main data }
    if fileexist(mdata) then
      setfile(datacopy,concat(strip(floppy), "\",hcodes,xcluster, fcodes[mainver  :1],  ".dat"));
      if fileexist(datacopy) then
        errmsg("***     %s     - ,  ",xcluster,floppy);
      endif;
      filecopy(mdata,datacopy);
      { !!! NETWORK !!! change wording of message if using a network }
      errmsg("    %s  ",xcluster);
    else
      errmsg("Main data file for cluster %s does not exist on this machine",xcluster);
    endif;
  else {if the verification data }
    if fileexist(vdata) then
      setfile(datacopy,concat(strip(floppy), "\",hcodes,xcluster, fcodes[mainver  :1],  ".dat"));
      if fileexist(datacopy) then
        errmsg("*** Verification data for cluster %s already exists on drive %s -- Please check",xcluster,floppy);
      endif;
      filecopy(vdata,datacopy);
      { !!! NETWORK !!! change wording of message if using a network }
      errmsg("Verification data file for cluster %s copied to diskette",xcluster);
    else
      errmsg("Verification data file for cluster %s does not exist on this machine",xcluster);
    endif;
  endif
end;

PROC ENTER_FF
preproc
CSPro="C:\Program Files\CSPro 2.6";
{ Country code }
cc="CC";
{ Project directory }
proj="MICS\CSPRO";
{ Entry application name }
app="ENTRY";
{ Dictionary name }
dic="MICS3";
{ Operator ID }
opid="MICS"; 		{ this could be an ID set on each machine }
{ Transfer drive (floppy by default) }
{ !!! NETWORK !!! if using network, change "A:" to shared directory }
floppy="S:\MICS";
{ File codes - M = Main, V = Verification,) }
fcodes = "MV";
hcodes = "HH";

nl = "newline";
ocodes = "AMT-RUV-NDQ";

{ Set work drive }
wrkdrv="C:";
wrkprj=concat(strip(wrkdrv), "\", PROJ);

{ Main and verification data will be in these directories }
data=concat(strip(wrkprj), "\DATA");
veri=concat(strip(wrkprj), "\VERI");

{ The following directories are assumed }
{ \MICS        Main survey directory (some files located here)}
{ \MICS\ENTRY  Data entry programs located here               }
{ \MICS\DICTS  Directory used to store dictionaries           }
{ \MICS\DATA   Directory used to store data                   }
{ \MICS\VERI   Directory used to store verification data      }

{ Environment variables used for default directory names (applies to both Entry & Supervisor computers) }
entry="Entry";
dicts="Dicts";

{ Check that all directories needed exist }
xdir = wrkprj; chkdir();
xdir = concat(strip(wrkprj), "\", strip(entry)); chkdir();
xdir = concat(strip(wrkprj), "\", strip(dicts)); chkdir();
xdir = data; chkdir();
xdir = veri; chkdir();

{ Check that there are not two concurrent sessions of CSPro loaded at the same time }
if fileexist(jstone) then
      errmsg("There is another data entry session running or the computer crashed while running");
  if accept(
            "Reset","Yes","No"
           ) = 1 then
    filedelete(jstone)
  else
    stop(-1);
  endif;
endif;
{ create the file justone [it is deleted when option 11, quit, is selected by the DE operator }
setfile(jstone,concat(strip(wrkprj),"\justone.txt"),create);

PROC ECLUSTER
{ check cluster is in the sample file }
recode ecluster => err;
        001-304 => 0;    { Give list of valid cluster numbers }
                => 1;
endrecode;
if err then
  display("Cluster number not valid");
  reenter;
endif;
xcluster = edit("999",$);
{ set cluster data file names }
setfile(mdata,concat(strip(data), "\",hcodes, xcluster, "M", ".DAT"));
setfile(vdata,concat(strip(veri), "\",hcodes, xcluster, "V", ".DAT"));
{ set ppf file name }
setfile(pfffile,concat(strip(wrkprj), "\", strip(entry), "\", strip(app), ".pff"));
{ set name of file that helps make sure that only one version of CSPRO runs at a time }
setfile(jstone,concat(strip(wrkprj),"\justone.txt"));
{ set name of the project directory }
setfile(dirname,wrkprj);
close(dirname);

always = 1;
while always do   { loop forever }
  { Get the selected option }
  { !!! NETWORK !!! replace "diskette" in options T and V with "network" }
  choice = accept("   ",
                  "A    ........................A",
                  "M   -...................M",
                  "T    -........T",
                  "------------------------------------------------",
                  "R     ....................R",
                  "U    .................U",
                  "V     ......V",
                  "------------------------------------------------",
                  "N    ........................N",
                  "Q  .......................................Q");

      if choice =  0 then {ESC} reenter; { escape from menu & allow entry of new cluster number }
  elseif choice =  1 then {ADD} csrun(1,1); {Main}
  elseif choice =  2 then {MOD} csrun(0,1); {Main}
  elseif choice =  3 then Transfer(1);		{Main}
  elseif choice =  4 then {---} { do nothing }
  elseif choice =  5 then {ADD} csrun(1,2); {Verification}
  elseif choice =  6 then {MOD} csrun(0,2); {Verification}
  elseif choice =  7 then Transfer(2);		{Verification}
  elseif choice =  8 then {---} { do nothing }
  elseif choice =  9 then {NEW} reenter; { get the new cluster number }
  elseif choice = 10 then {QUIT}
    if fileexist(jstone) then
      filedelete(jstone);
    endif;
    stop(-1);       { quit the process }
  endif;
enddo;
