PROC GLOBAL
set explicit;
{ Working variables -- check the lengths needed for all fields }
alpha(32) CSPro;			{ CSPro executables directory }
alpha(2)  cc;				{ country code }
alpha(26)  proj;				{ project directory name }
alpha(8)  cmapp, deapp, edapp, exapp, gpapp, scapp, dic; { application names and dictionary name }
alpha(16) clusterapp;
alpha(20) printapp;
alpha(8)  opid, nl;			{ operator ID, "newline" string }
alpha(30)  workdrive;
alpha(2)  wrkdrv;               { floppy and workdrive letters, e.g. "A:", "C:" }
alpha(32) wrkprj, backup, expdir, gps, final, raw, veri, xdir;     { directories }
alpha(8)  dicts, entry, super;          { directory names for DICTS, ENTRY, SUPER -- suggest using these names }
alpha(3)  xcluster;                     { cluster number -- make sure length is correct!!! }
alpha(2)  fcodes;                       { file codes - M = Main, V = Verification }
alpha(12) ocodes;                       { option codes AMT-RUV-NDQ }
alpha(2)  hcodes;                       { code for household data file name }
alpha(1)  frstrun;						{ is this the first run of the editing program }

numeric always, choice, chkok, noyes, err, docopy, t;
numeric flowentryopid, skippingfirstpart, skippingentryop, i, maxclusternumber, startpos;

file pfffile mdata vdata wdata datacopy alldata jstone dirname;

{ ---------------------------------------------------------------------------- }

function chkdir()
  setfile(dirname,xdir);
  if fileexist(xdir) <> 1 then
    errmsg("Directory %s does not exist",xdir);
    stop(-1);
  endif;
end;

function chkexist(mychoice)
  chkok = 1;
  if mychoice = 4 then { if verifying data }
    { !!! NETWORK !!! change wording of message if using a network }
    errmsg("*** Insert diskette containing verification data in drive %s",workdrive);
    setfile(vdata,concat(strip(workdrive), "\",hcodes, xcluster, "V", ".DAT"));
    if !fileexist(wdata) then { main data doesn't exist in raw directory }
      errmsg("Raw data file for cluster %s does not exist --- use option %s first",xcluster,"A");
      chkok = 0;
    elseif !fileexist(vdata) then { verification data doesn't exist }
      { !!! NETWORK !!! change wording of message if using a network }
      errmsg("Verification data file for cluster %s is not on the diskette",xcluster);
      chkok = 0;
    endif;
  elseif mychoice = 5 then { if backing up raw data }
    if !fileexist(wdata) then { main data doesn't exist in raw directory }
      errmsg("Raw data file for cluster %s does not exist",xcluster);
      chkok = 0;
    endif;
  elseif mychoice in 7,8,9 then { if using raw data }
    if !fileexist(wdata) then { raw data doesn't exist }
      errmsg("Raw data file for cluster %s does not exist",xcluster);
      chkok = 0;
    endif;
  endif;
  chkexist = chkok;
end;

function genpff(mychoice)
  if mychoice = 3 then { structure check }
    setfile(pfffile,concat(strip(wrkprj), "\", strip(super), "\", strip(scapp), ".pff"));
    filecreate(pfffile);
    filewrite(pfffile, "[Run Information]");
    filewrite(pfffile, "Version=CSPro 2.6");
    filewrite(pfffile, "AppType=Batch");
    filewrite(pfffile, "");
    filewrite(pfffile, "[Files]");
    filewrite(pfffile, "Application=.\check.bch");
    filewrite(pfffile, concat("InputData=", strip(raw), "\", hcodes, xcluster,"M", ".DAT"));
    filewrite(pfffile, "Listing=.\check.lst");
    filewrite(pfffile, "");
    filewrite(pfffile, "[Parameters]");
    filewrite(pfffile, "ViewListing=Always");
  elseif mychoice = 4 then { verification }
    setfile(pfffile,concat(strip(wrkprj), "\", strip(super), "\", strip(cmapp), ".cmp.pff"));
    filecreate(pfffile);
    filewrite(pfffile, "[Run Information]");
    filewrite(pfffile, "Version=CSPro 2.6");
    filewrite(pfffile, "AppType=Compare");
    filewrite(pfffile, "");
    filewrite(pfffile, "[Files]");
    filewrite(pfffile, "Application=.\compare.cmp");
    filewrite(pfffile, concat("InputData=", strip(raw), "\", hcodes, xcluster,"M", ".DAT"));
    filewrite(pfffile, "Listing=.\compare.cmp.lst");
    filewrite(pfffile, concat("ReferenceData=", strip(workdrive), "\", hcodes, xcluster,"V", ".DAT"));
    filewrite(pfffile, "");
    filewrite(pfffile, "[Parameters]");
    filewrite(pfffile, "ViewListing=Always");
  elseif mychoice = 7 then { editing }
  	noyes = accept("First Run of Editing Program for this Cluster","No","Yes");
  	frstrun = edit("9",noyes-1);
    setfile(pfffile,concat(strip(wrkprj), "\", strip(super), "\", strip(edapp), ".pff"));
    filecreate(pfffile);
    filewrite(pfffile, "[Run Information]");
    filewrite(pfffile, "Version=CSPro 2.6");
    filewrite(pfffile, "AppType=Batch");
    filewrite(pfffile, "");
    filewrite(pfffile, "[Files]");
    filewrite(pfffile, "Application=.\editing.bch");
    filewrite(pfffile, concat("InputData=", strip(raw), "\", hcodes, xcluster,"M", ".DAT"));
    filewrite(pfffile, "Listing=.\editing.lst");
    filewrite(pfffile, "");
    filewrite(pfffile, "[Parameters]");
    filewrite(pfffile, "ViewListing=Always");
    filewrite(pfffile, concat("Parameter=",frstrun));
  elseif mychoice = 8 then { data entry }
    setfile(pfffile,concat(strip(wrkprj), "\", strip(entry), "\", strip(deapp), ".pff"));
    filecreate(pfffile);
    filewrite(pfffile, "[Run Information]");
    filewrite(pfffile, "Version=CSPro 2.6");
    filewrite(pfffile, "AppType=Entry");
    filewrite(pfffile, "");
    filewrite(pfffile, "[DataEntryInit]");
    filewrite(pfffile, concat("OperatorID=", strip(opid)));
    filewrite(pfffile, "Lock=Add, Verify");
    filewrite(pfffile, "Fullscreen=No");
    filewrite(pfffile, "NoFileOpen=Yes");
    filewrite(pfffile, "");
    filewrite(pfffile, "[Files]");
    filewrite(pfffile, concat("Application=", strip(wrkprj), "\", strip(entry), "\", strip(deapp), ".ent"));
    filewrite(pfffile, concat("InputData=", strip(raw), "\", hcodes, xcluster,"M", ".DAT"));
    filewrite(pfffile, "");
    filewrite(pfffile, "[Parameters]");
    filewrite(pfffile, concat("Parameter=", xcluster));
  elseif mychoice = 11 then { export to SPSS }
    setfile(pfffile,concat(strip(wrkprj), "\", strip(expdir), "\", strip(exapp), ".pff"));
    filecreate(pfffile);
    filewrite(pfffile, "[Run Information]");
    filewrite(pfffile, "Version=CSPro 2.6");
    filewrite(pfffile, "AppType=Batch");
    filewrite(pfffile, "");
    filewrite(pfffile, "[Files]");
    filewrite(pfffile, "Application=..\export\export.bch");
    filewrite(pfffile, concat("InputData=", strip(final), "\all.dat"));
    filewrite(pfffile, "Listing=..\export\export.lst");
    filewrite(pfffile, "");
    filewrite(pfffile, "[UserFiles]");
    filewrite(pfffile, "MYCH=..\..\SPSS\MYCH.DAT");
    filewrite(pfffile, "MYHH=..\..\SPSS\MYHH.DAT");
    filewrite(pfffile, "MYHL=..\..\SPSS\MYHL.DAT");
    filewrite(pfffile, "MYWM=..\..\SPSS\MYWM.DAT");
    filewrite(pfffile, "");
    filewrite(pfffile, "[Parameters]");
    filewrite(pfffile, "ViewListing=Always");
  elseif mychoice = 13 or mychoice = 14 then { enter or modify GPS data }
    setfile(pfffile,concat(strip(wrkprj), "\", strip(gps), "\", strip(gpapp), ".pff"));
    filecreate(pfffile);
    filewrite(pfffile, "[Run Information]");
    filewrite(pfffile, "Version=CSPro 2.6");
    filewrite(pfffile, "AppType=Entry");
    filewrite(pfffile, "");
    filewrite(pfffile, "[DataEntryInit]");
    filewrite(pfffile, concat("OperatorID=", strip(opid)));
    if mychoice = 13 then { add data }
      filewrite(pfffile, "Lock=Modify,Verify");
    elseif mychoice = 14 then { modify data }
      filewrite(pfffile, "Lock=Add,Verify");
    endif;
    filewrite(pfffile, "Fullscreen=No");
    filewrite(pfffile, "NoFileOpen=Yes");
    filewrite(pfffile, "");
    filewrite(pfffile, "[Files]");
    filewrite(pfffile, concat("Application=", strip(wrkprj), "\", strip(gps), "\", strip(gpapp), ".ent"));
    filewrite(pfffile, concat("InputData=","..\", strip(gps), "\gps.dat"));
    filewrite(pfffile, "");
  elseif mychoice = 17 then { print cluster tracking info check }
    setfile(pfffile,concat(strip(wrkprj), "\", strip(super), "\", strip(printapp), ".pff"));
    filecreate(pfffile);
    filewrite(pfffile, "[Run Information]");
    filewrite(pfffile, "Version=CSPro 2.6");
    filewrite(pfffile, "AppType=Batch");
    filewrite(pfffile, "");
    filewrite(pfffile, "[Files]");
    filewrite(pfffile, "Application=.\print_tracking_info.bch");
    filewrite(pfffile, concat("InputData=", strip(wrkprj), "\", strip(super), "\", strip(clusterapp), ".DAT"));
    filewrite(pfffile, "Listing=.\print_tracking_info.lst");
    filewrite(pfffile, "");
    filewrite(pfffile, "[Parameters]");
    filewrite(pfffile, "ViewListing=Always");
  endif;
  
  close(pfffile);
end;

function runpff(mychoice);
  if mychoice = 1 then { cluster info entry }
    Execsystem(concat('"', strip(cspro), '\CSEntry.exe" ',
                      strip(wrkprj), "\", strip(super), "\", strip(clusterapp), '.pff'),
               maximized,wait);
  elseif mychoice = 3 then { structure checking }
    { Using single quotes instad of double quotes as double quotes are needed in the string generated }
    Execsystem(concat('"', strip(cspro), '\CSBatch.exe" ',
                      strip(wrkprj), "\", strip(super), "\", strip(scapp), '.pff'),
               maximized,wait);
    t = accept("  ?", "1 ", "2 ");
    if t = 1 then
    	tcluster = ecluster;
		t = loadcase(cluster_tracking_dict, tcluster);
	    if !t then 
	    	{ do nothing  }
			errmsg("Cluster tracking information not entered");
		else
			TCHECK = sysdate("DDMMYYYY");
		    t = writecase(cluster_tracking_dict, tcluster);
		    close(cluster_tracking_dict);
    	endif;	
    endif;
  elseif mychoice = 4 then { verification }
    Execsystem(concat('"', strip(cspro), '\CSDiff.exe" ',
                      strip(wrkprj), "\", strip(super), "\", strip(cmapp), '.cmp.pff'),
               maximized,wait);
    t = accept("Verification complete?", "1 Yes", "2 No");
    if t = 1 then
    	tcluster = ecluster;
		t = loadcase(cluster_tracking_dict, tcluster);
	    if !t then 
	    	{ do nothing  }
			errmsg("Cluster tracking information not entered");
		else
			TVERI = sysdate("DDMMYYYY");
		    t = writecase(cluster_tracking_dict, tcluster);
		    close(cluster_tracking_dict);
    	endif;	
    endif;
  elseif mychoice = 7 then { editing }
    Execsystem(concat('"', strip(cspro), '\CSBatch.exe" ',
                      strip(wrkprj), "\", strip(super), "\", strip(edapp), '.pff'),
               maximized,wait);
	t = accept("Editing complete?", "1 Yes", "2 No");
    if t = 1 then
    	tcluster = ecluster;
		t = loadcase(cluster_tracking_dict, tcluster);
	    if !t then 
	    	{ do nothing }
			errmsg("Cluster tracking information not entered");
		else
			TEDIT = sysdate("DDMMYYYY");
		    t = writecase(cluster_tracking_dict, tcluster);
		    close(cluster_tracking_dict);
    	endif;	
    endif;

  elseif mychoice = 8 then { data entry }
    Execsystem(concat('"', strip(cspro), '\CSEntry.exe" ',
                      strip(wrkprj), "\", strip(entry), "\", strip(deapp), '.pff'),
               maximized,wait);
  elseif mychoice = 11 then { export to SPSS }
    Execsystem(concat('"', strip(cspro), '\CSBatch.exe" ',
                      strip(wrkprj), "\", strip(expdir), "\", strip(exapp), '.pff'),
               maximized,wait);
  elseif mychoice = 13 or mychoice = 14 then { GPS data entry }
    Execsystem(concat('"', strip(cspro), '\CSEntry.exe" ',
                      strip(wrkprj), "\", strip(gps), "\", strip(gpapp), '.pff'),
               maximized,wait);
  elseif mychoice = 17 then { print cluster tracking info }
    { Using single quotes instad of double quotes as double quotes are needed in the string generated }
    Execsystem(concat('"', strip(cspro), '\CSBatch.exe" ',
                      strip(wrkprj), "\", strip(super), "\", strip(printapp), '.pff'),
               maximized,wait);
  endif;
end;

function csrun(mychoice)
  if chkexist(mychoice) then
    genpff(mychoice);
    runpff(mychoice);
  endif;
end;

function transfer(mychoice)
  if mychoice = 3 then { if checking structure }
    if fileexist(wdata) then
      docopy = accept("   ?","","")-1;
    else
      docopy = 1;
    endif;
    if docopy then
      { !!! NETWORK !!! remove message if using a network}
      errmsg("***         %s",workdrive);
      setfile(mdata,concat(strip(workdrive), "\",hcodes, xcluster, "M", ".DAT"));
      if !fileexist(mdata) then { if main data not on diskette }
        { !!! NETWORK !!! change wording of message if using a network }
        errmsg("     %s   ",xcluster);
        chkok = 0;
      else
        filecopy(mdata,wdata);
        errmsg("     %s    \RAW",xcluster);
        genpff(mychoice);
        runpff(mychoice);
      endif;
    else
      genpff(mychoice);
      runpff(mychoice);
    endif;
  elseif mychoice = 5 then
    if chkexist(mychoice) then
      { copy the data to the backup directory }
      setfile(datacopy,concat(strip(backup), "\",hcodes,xcluster, "M",  ".dat"));
      filecopy(wdata,datacopy);
      errmsg("     %s    \BACKUP",xcluster);

	  { Write cluster tracking info }
	  tcluster = ecluster;
	  t = loadcase(cluster_tracking_dict, tcluster);
	  if !t then 
	   	{ do nothing   }
		errmsg("Cluster tracking information not entered");
	  else
		TRAW = sysdate("DDMMYYYY");
	    t = writecase(cluster_tracking_dict, tcluster);
	    close(cluster_tracking_dict);
	  endif;	
    endif;
  elseif mychoice = 9 then
    if chkexist(mychoice) then
      { copy the data to the final directory }
      setfile(datacopy,concat(strip(final), "\",hcodes,xcluster, "M",  ".dat"));
      filecopy(wdata,datacopy);
      errmsg("     %s    \FINAL",xcluster);

	  { Write cluster tracking info }
      tcluster = ecluster;
	  t = loadcase(cluster_tracking_dict, tcluster);
	  if !t then 
	   	{ do nothing   }
		errmsg("Cluster tracking information not entered");
	  else
		TFINAL = sysdate("DDMMYYYY");
	    t = writecase(cluster_tracking_dict, tcluster);
	    close(cluster_tracking_dict);
	  endif;	
    endif;
  elseif mychoice = 11 then
    { concatonate the cluster data }
    setfile(alldata,concat(strip(final), "\all.dat"));
    fileconcat(alldata,concat(strip(final),"\",hcodes,"*M.dat"));
    csrun(mychoice);
  endif;
end;


PROC SUPER_FF
preproc
CSPro="C:\Program Files\CSPro 2.6";
{ Country code }
cc="cc";
{ Project directory }
proj="MICS\CSPRO";
{ Compare application name }
cmapp="COMPARE";
{ Entry application name }
deapp="ENTRY";
{ Editing application name }
edapp="EDITING";
{ Export application name }
exapp="EXPORT";
{ GPS application name }
gpapp="GPSENTRY";
{ Structure check application name }
scapp="CHECK";
{ Cluster info entry application name }
clusterapp="CLUSTER_TRACKING";
{ Cluster info printing application name }
printapp="PRINT_TRACKING_INFO";
{ Dictionary name }
dic="MICS3";
{ Operator ID }
opid="MICS"; 		{ this could be an ID set on each machine }
{ Transfer drive (floppy by default) }
{ !!! NETWORK !!! if using network, change "A:" to shared directory }
workdrive="S:\MICS\Mihaleva";
{ File codes - M = Main, V = Verification,) }
fcodes = "MV";
hcodes = "HH";
{ Number of clusters  }
maxclusternumber = 304;

nl = "newline";
ocodes = "AMT-RUV-NDQ";

{ Set work drive }
wrkdrv="C:";
wrkprj=concat(strip(wrkdrv), "\", PROJ);

{ Create data directories }
raw=concat(strip(wrkprj), "\RAW");
backup=concat(strip(wrkprj), "\BACKUP");
final=concat(strip(wrkprj), "\FINAL");

{ The following directories are assumed }
{ \MICS        Main survey directory (some files located here)}
{ \MICS\BACKUP Directory used to store backup data            }
{ \MICS\DICTS  Directory used to store dictionaries           }
{ \MICS\ENTRY  Data entry programs located here               }
{ \MICS\EXPORT Directory used to store export programs        }
{ \MICS\FINAL  Directory used to store final data             }
{ \MICS\GPS    Directory used to store GPS data               }
{ \MICS\RAW    Directory used to store raw data               }

{ Environment variables used for default directory names (applies to both Entry & Supervisor computers) }
dicts="Dicts";
entry="Entry";
expdir="Export";
gps="GPS";
super="Super";

{ Check that all directories needed exist }
xdir = wrkprj; chkdir();
xdir = concat(strip(wrkprj), "\", strip(dicts)); chkdir();
xdir = concat(strip(wrkprj), "\", strip(entry)); chkdir();
xdir = concat(strip(wrkprj), "\", strip(expdir)); chkdir();
xdir = concat(strip(wrkprj), "\", strip(gps)); chkdir();
xdir = concat(strip(wrkprj), "\", strip(super)); chkdir();
xdir = backup; chkdir();
xdir = final; chkdir();
xdir = raw; chkdir();

{ Check that there are not two concurrent sessions of CSPro loaded at the same time }
if fileexist(jstone) then
      errmsg("There is another data entry session running or the computer crashed while running");
  if accept(
            "Reset","Yes","No"
           ) = 1 then
    filedelete(jstone)
  else
    stop(-1);
  endif;
endif;
{ create the file justone [it is deleted when option 11, quit, is selected by the DE operator }
setfile(jstone,concat(strip(wrkprj),"\justone.txt"),create);

PROC ECLUSTER
{ check cluster is in the sample file }
recode ecluster => err;
        001-304 => 0;    { Give list of valid cluster numbers }
                => 1;
endrecode;
if err then
  display("   ");
  reenter;
endif;
xcluster = edit("999",$);
{ set location of raw data }
setfile(wdata,concat(strip(raw), "\",hcodes, xcluster, "M", ".DAT"));
{ set name of file that helps make sure that only one version of CSPRO runs at a time }
setfile(jstone,concat(strip(wrkprj),"\justone.txt"));
{ set name of the project directory }
setfile(dirname,wrkprj);
close(dirname);

always = 1;
while always do   { loop forever }
  { Get the selected option }
  choice = accept(" ",
                  "   .................",
                  "---------------------------------------------",
                  "  ....................",
                  " ..............................",
                  "   ........",
                  "---------------------------------------------",
                  " ................",
                  " .............................",
                  "   ...",
                  "---------------------------------------------",
                  "   SPSS ( ).........",
                  "---------------------------------------------",
                  "  .........................",
                  "........................................");

	if choice = 1 then
	    tcluster = ecluster;
	    t = loadcase(cluster_tracking_dict, tcluster);
		{ If we're not able to load the case, initialize data file }
	    if !t then 
			display("Initialize file");
	    	{ set whole record to zero }
	    	TRECV = 0; TNUMHHSEL = 0; TNUMHHCOMP = 0; TNUMHHQUEST = 0;
	    	TNUMWMQUEST = 0; TNUMWMCOMP = 0; TNUMCHQUEST = 0; TNUMCHCOMP = 0;
	    	TENTRYOPID = 0; TENTRY = 0; TVENTRYOPID = 0; TVENTRY = 0; TVERI = 0; 
	    	TCHECK = 0; TRAW = 0; TEDIT = 0; TFINAL = 0; 
			i = 1;
			{ For each cluster }
			while i < maxclusternumber do
				{ Check if i is a valid cluster number, if not skip to next }
				if not i in 1:304 then {  !!! Change to your clusternumber range }
					i = i + 1;
					next; 
				endif;
				tcluster = i;
				{ Write information for this cluster to file }
			    t = writecase(cluster_tracking_dict, tcluster); 
			    if !t then 
			    	errmsg("Not able to create cluster tracking data file"); 
			    endif;
			    i = i + 1;
			enddo;
			tcluster = ecluster;
		endif;
		startpos = accept("  :", 
						  "1     ",
						  "2   ", 
						  "3     ");
		{ Check what the user wants to do }
		enter cluster_tracking_ff;
    	tcluster = ecluster;
	    t = writecase(cluster_tracking_dict, tcluster); 
	    if !t then 
	    	errmsg("Not able write information about cluster %03d"); 
	    endif; 
	    close(cluster_tracking_dict);
	endif;

      if choice =  0 then {ESC} reenter; { escape from menu & allow entry of new cluster number }
  elseif choice =  3 then {A} transfer(choice);   { Structure Check             }
  elseif choice =  4 then {B} csrun(choice);      { Verification                }
  elseif choice =  5 then {C} transfer(choice);		{ Backup Raw Data             }
  elseif choice =  6 then {---}                   { do nothing                  }
  elseif choice =  7 then {D} csrun(choice);      { Editing                     }
  elseif choice =  8 then {E} csrun(choice);      { Modify the Data             }
  elseif choice =  9 then {F} transfer(choice);		{ Backup Final Data           }
  elseif choice =  10 then {---}                   { do nothing                  }
  elseif choice =  11 then {G} transfer(choice);   { Export to SPSS              }
  elseif choice =  12 then {---}                  { do nothing                  }
  elseif choice =  13 then {NEW} reenter;         { get the new cluster number  }
  elseif choice =  14 then {QUIT}
    if fileexist(jstone) then
      filedelete(jstone);
    endif;
    stop(-1);       { quit the process }
  endif;
enddo;

PROC TINFO_FORM
if ecluster = 0 then
	{  }
endif;

PROC TRECVD
preproc 
if startpos <> 1 then
	advance TENTRYOPID;
endif;

PROC TNUMCHCOMP
if startpos = 1 then
	advance; 
endif;

PROC TENTRYOPID
preproc
if startpos <> 2 then
	advance TVENTRYOPID;
endif;

PROC TENTRYY
if startpos = 2 then
	advance; 
endif;
